@@ -29,6 +29,7 @@
                                             "create-function"
                                             "create-test"
                                             "cycle-coll"
+                                            "cycle-keyword-auto-resolve"
                                             "cycle-privacy"
                                             "demote-fn"
                                             "destructure-keys"
@@ -103,10 +104,11 @@
                        :serverInfo true
                        :clojuredocs true}}}
 
-      (lsp/request! (fixture/initialize-request))))
+      (lsp/request! (fixture/initialize-request (merge fixture/default-init-options
+                                                       {:workDoneToken "integration-test"})))))
 
   (h/assert-submap
-    {:token "clojure-lsp"
+    {:token "integration-test"
      :value {:kind "begin"
              :title "clojure-lsp"
              :percentage 0}}
