@@ -18,16 +18,32 @@
    [lsp4clj.liveness-probe :as lsp.liveness-probe]
    [lsp4clj.lsp.requests :as lsp.requests]
    [lsp4clj.server :as lsp.server]
+   [medley.core :as medley]
+   [promesa.core :as p]
+   [promesa.exec :as p.exec]
    [taoensso.timbre :as timbre]))
 
 (set! *warn-on-reflection* true)
 
+(defmacro eventually [& body]
+  `(p/thread ~@body))
+
+(def ^:private changes-executor (memoize p.exec/forkjoin-executor))
+
+(defmacro after-changes [& body]
+  `(p/thread-call (changes-executor) (^:once fn* [] ~@body)))
+
 (def diagnostics-debounce-ms 100)
 (def change-debounce-ms 300)
 (def watched-files-debounce-ms 1000)
 
 (def known-files-pattern "**/*.{clj,cljs,cljc,cljd,edn,bb,clj_kondo}")
 
+(def normalize-uri shared/normalize-uri-from-client)
+
+(defn normalize-doc-uri [params]
+  (medley/update-existing-in params [:text-document :uri] normalize-uri))
+
 (defn log! [level args fmeta]
   (timbre/log! level :p args {:?line (:line fmeta)
                               :?file (:file fmeta)
@@ -83,7 +99,6 @@
 
   (publish-diagnostic [_this diagnostic]
     (lsp.server/discarding-stdout
-      (logger/debug (format "Publishing %s diagnostics for %s" (count (:diagnostics diagnostic)) (:uri diagnostic)))
       (shared/logging-task
         :publish-diagnostics
         (->> diagnostic
@@ -116,7 +131,7 @@
   (publish-progress [_this percentage message progress-token]
     (lsp.server/discarding-stdout
       ;; ::coercer/notify-progress
-      (->> (lsp.requests/work-done-progress percentage message (or progress-token "clojure-lsp"))
+      (->> (lsp.requests/work-done-progress percentage message progress-token)
            (lsp.server/send-notification server "$/progress"))))
 
   (show-message-request [_this message type actions]
@@ -156,12 +171,15 @@
 ;;;; clojure extra features
 
 (defmethod lsp.server/receive-request "clojure/dependencyContents" [_ components params]
-  (->> params
+  (->> (medley/update-existing params :uri normalize-uri)
        (handler/dependency-contents components)
-       (conform-or-log ::coercer/uri)))
+       (conform-or-log ::coercer/uri)
+       eventually))
 
 (defmethod lsp.server/receive-request "clojure/serverInfo/raw" [_ components _params]
-  (handler/server-info-raw components))
+  (->> components
+       handler/server-info-raw
+       eventually))
 
 (defmethod lsp.server/receive-notification "clojure/serverInfo/log" [_ components _params]
   (future
@@ -172,163 +190,202 @@
         (throw e)))))
 
 (defmethod lsp.server/receive-request "clojure/cursorInfo/raw" [_ components params]
-  (handler/cursor-info-raw components params))
+  (->> params
+       (normalize-doc-uri)
+       (handler/cursor-info-raw components)
+       eventually))
 
 (defmethod lsp.server/receive-notification "clojure/cursorInfo/log" [_ components params]
   (future
     (try
-      (handler/cursor-info-log components params)
+      (handler/cursor-info-log components (normalize-doc-uri params))
       (catch Throwable e
         (logger/error e)
         (throw e)))))
 
 (defmethod lsp.server/receive-request "clojure/clojuredocs/raw" [_ components params]
-  (handler/clojuredocs-raw components params))
+  (->> params
+       (handler/clojuredocs-raw components)
+       eventually))
 
 ;;;; Document sync features
 
 ;; https://microsoft.github.io/language-server-protocol/specifications/lsp/3.17/specification/#textDocument_synchronization
 
 (defmethod lsp.server/receive-notification "textDocument/didOpen" [_ components params]
-  (handler/did-open components params))
+  (handler/did-open components (normalize-doc-uri params)))
 
 (defmethod lsp.server/receive-notification "textDocument/didChange" [_ components params]
-  (handler/did-change components params))
+  (handler/did-change components (normalize-doc-uri params)))
 
 (defmethod lsp.server/receive-notification "textDocument/didSave" [_ components params]
   (future
     (try
-      (handler/did-save components params)
+      (handler/did-save components (normalize-doc-uri params))
       (catch Throwable e
         (logger/error e)
         (throw e)))))
 
 (defmethod lsp.server/receive-notification "textDocument/didClose" [_ components params]
-  (handler/did-close components params))
+  (handler/did-close components (normalize-doc-uri params)))
 
 (defmethod lsp.server/receive-request "textDocument/references" [_ components params]
   (->> params
+       (normalize-doc-uri)
        (handler/references components)
-       (conform-or-log ::coercer/locations)))
+       (conform-or-log ::coercer/locations)
+       eventually))
 
 (defmethod lsp.server/receive-request "textDocument/completion" [_ components params]
   (->> params
+       (normalize-doc-uri)
        (handler/completion components)
-       (conform-or-log ::coercer/completion-items)))
+       (conform-or-log ::coercer/completion-items)
+       eventually))
 
 (defmethod lsp.server/receive-request "completionItem/resolve" [_ components item]
   (->> item
        (conform-or-log ::coercer/input.completion-item)
        (handler/completion-resolve-item components)
-       (conform-or-log ::coercer/completion-item)))
+       (conform-or-log ::coercer/completion-item)
+       eventually))
 
 (defmethod lsp.server/receive-request "textDocument/prepareRename" [_ components params]
   (->> params
+       (normalize-doc-uri)
        (handler/prepare-rename components)
-       (conform-or-log ::coercer/prepare-rename-or-error)))
+       (conform-or-log ::coercer/prepare-rename-or-error)
+       after-changes))
 
 (defmethod lsp.server/receive-request "textDocument/rename" [_ components params]
   (->> params
+       (normalize-doc-uri)
        (handler/rename components)
-       (conform-or-log ::coercer/workspace-edit-or-error)))
+       (conform-or-log ::coercer/workspace-edit-or-error)
+       eventually))
 
 (defmethod lsp.server/receive-request "textDocument/hover" [_ components params]
   (->> params
+       (normalize-doc-uri)
        (handler/hover components)
-       (conform-or-log ::coercer/hover)))
+       (conform-or-log ::coercer/hover)
+       eventually))
 
 (defmethod lsp.server/receive-request "textDocument/signatureHelp" [_ components params]
   (->> params
+       (normalize-doc-uri)
        (handler/signature-help components)
-       (conform-or-log ::coercer/signature-help)))
+       (conform-or-log ::coercer/signature-help)
+       eventually))
 
 (defmethod lsp.server/receive-request "textDocument/formatting" [_ components params]
   (->> params
+       (normalize-doc-uri)
        (handler/formatting components)
-       (conform-or-log ::coercer/edits)))
-
-(def ^:private formatting (atom false))
+       (conform-or-log ::coercer/edits)
+       eventually))
 
 (defmethod lsp.server/receive-request "textDocument/rangeFormatting" [_this components params]
-  (when (compare-and-set! formatting false true)
-    (try
-      (->> params
-           (handler/range-formatting components)
-           (conform-or-log ::coercer/edits))
-      (catch Exception e
-        (logger/error e))
-      (finally
-        (reset! formatting false)))))
+  (->> params
+       (normalize-doc-uri)
+       (handler/range-formatting components)
+       (conform-or-log ::coercer/edits)
+       after-changes))
 
 (defmethod lsp.server/receive-request "textDocument/codeAction" [_ components params]
   (->> params
+       (normalize-doc-uri)
        (handler/code-actions components)
-       (conform-or-log ::coercer/code-actions)))
+       (conform-or-log ::coercer/code-actions)
+       after-changes))
 
 (defmethod lsp.server/receive-request "textDocument/codeLens" [_ components params]
   (->> params
+       (normalize-doc-uri)
        (handler/code-lens components)
-       (conform-or-log ::coercer/code-lenses)))
+       (conform-or-log ::coercer/code-lenses)
+       after-changes))
 
-(defmethod lsp.server/receive-request "codeLens/resolve" [_ components params]
-  (->> params
+(defmethod lsp.server/receive-request "codeLens/resolve" [_ components code-lens]
+  (->> code-lens
        (handler/code-lens-resolve components)
-       (conform-or-log ::coercer/code-lens)))
+       (conform-or-log ::coercer/code-lens)
+       eventually))
 
 (defmethod lsp.server/receive-request "textDocument/definition" [_ components params]
   (->> params
+       (normalize-doc-uri)
        (handler/definition components)
-       (conform-or-log ::coercer/location)))
+       (conform-or-log ::coercer/location)
+       eventually))
 
 (defmethod lsp.server/receive-request "textDocument/declaration" [_ components params]
   (->> params
+       (normalize-doc-uri)
        (handler/declaration components)
-       (conform-or-log ::coercer/location)))
+       (conform-or-log ::coercer/location)
+       eventually))
 
 (defmethod lsp.server/receive-request "textDocument/implementation" [_ components params]
   (->> params
+       (normalize-doc-uri)
        (handler/implementation components)
-       (conform-or-log ::coercer/locations)))
+       (conform-or-log ::coercer/locations)
+       eventually))
 
 (defmethod lsp.server/receive-request "textDocument/documentSymbol" [_ components params]
   (->> params
+       (normalize-doc-uri)
        (handler/document-symbol components)
-       (conform-or-log ::coercer/document-symbols)))
+       (conform-or-log ::coercer/document-symbols)
+       after-changes))
 
 (defmethod lsp.server/receive-request "textDocument/documentHighlight" [_ components params]
   (->> params
+       (normalize-doc-uri)
        (handler/document-highlight components)
-       (conform-or-log ::coercer/document-highlights)))
+       (conform-or-log ::coercer/document-highlights)
+       after-changes))
 
 (defmethod lsp.server/receive-request "textDocument/semanticTokens/full" [_ components params]
   (->> params
+       (normalize-doc-uri)
        (handler/semantic-tokens-full components)
-       (conform-or-log ::coercer/semantic-tokens)))
+       (conform-or-log ::coercer/semantic-tokens)
+       after-changes))
 
 (defmethod lsp.server/receive-request "textDocument/semanticTokens/range" [_ components params]
   (->> params
+       (normalize-doc-uri)
        (handler/semantic-tokens-range components)
-       (conform-or-log ::coercer/semantic-tokens)))
+       (conform-or-log ::coercer/semantic-tokens)
+       after-changes))
 
 (defmethod lsp.server/receive-request "textDocument/prepareCallHierarchy" [_ components params]
   (->> params
+       (normalize-doc-uri)
        (handler/prepare-call-hierarchy components)
-       (conform-or-log ::coercer/call-hierarchy-items)))
+       (conform-or-log ::coercer/call-hierarchy-items)
+       eventually))
 
 (defmethod lsp.server/receive-request "callHierarchy/incomingCalls" [_ components params]
   (->> params
        (handler/call-hierarchy-incoming components)
-       (conform-or-log ::coercer/call-hierarchy-incoming-calls)))
+       (conform-or-log ::coercer/call-hierarchy-incoming-calls)
+       eventually))
 
 (defmethod lsp.server/receive-request "callHierarchy/outgoingCalls" [_ components params]
   (->> params
        (handler/call-hierarchy-outgoing components)
-       (conform-or-log ::coercer/call-hierarchy-outgoing-calls)))
+       (conform-or-log ::coercer/call-hierarchy-outgoing-calls)
+       eventually))
 
 (defmethod lsp.server/receive-request "textDocument/linkedEditingRange" [_ components params]
   (->> params
+       (normalize-doc-uri)
        (handler/linked-editing-ranges components)
-       (conform-or-log ::coercer/linked-editing-ranges-or-error)))
+       (conform-or-log ::coercer/linked-editing-ranges-or-error)
+       eventually))
 
 ;;;; Workspace features
 
@@ -347,19 +404,31 @@
   (logger/warn params))
 
 (defmethod lsp.server/receive-notification "workspace/didChangeWatchedFiles" [_ components params]
-  (->> params
+  (->> (medley/update-existing params
+                               :changes (fn [changes]
+                                          (mapv (fn [change]
+                                                  (medley/update-existing change :uri normalize-uri))
+                                                changes)))
        (conform-or-log ::coercer/did-change-watched-files-params)
        (handler/did-change-watched-files components)))
 
 (defmethod lsp.server/receive-request "workspace/symbol" [_ components params]
   (->> params
        (handler/workspace-symbols components)
-       (conform-or-log ::coercer/workspace-symbols)))
+       (conform-or-log ::coercer/workspace-symbols)
+       eventually))
 
 (defmethod lsp.server/receive-request "workspace/willRenameFiles" [_ components params]
-  (->> params
+  (->> (medley/update-existing params
+                               :files (fn [files]
+                                        (mapv (fn [file]
+                                                (-> file
+                                                    (medley/update-existing :old-uri normalize-uri)
+                                                    (medley/update-existing :new-uri normalize-uri)))
+                                              files)))
        (handler/will-rename-files components)
-       (conform-or-log ::coercer/workspace-edit)))
+       (conform-or-log ::coercer/workspace-edit)
+       after-changes))
 
 (defn capabilities [settings]
   (conform-or-log
@@ -425,10 +494,12 @@
   ;; about which messages are sent when probably needs to be handled in lsp4clj.
   ;; https://microsoft.github.io/language-server-protocol/specifications/lsp/3.17/specification/#initialize
   (handler/initialize components
-                      (:root-uri params)
+                      (normalize-uri (:root-uri params))
                       (:capabilities params)
                       (client-settings params)
                       (some-> params :work-done-token str))
+  (when-let [trace-level (:trace params)]
+    (lsp.server/set-trace-level server trace-level))
   (when-let [parent-process-id (:process-id params)]
     (lsp.liveness-probe/start! parent-process-id log-wrapper-fn #(exit server)))
   {:capabilities (capabilities (settings/all @db*))})
@@ -448,14 +519,18 @@
 (defmethod lsp.server/receive-notification "exit" [_ {:keys [server]} _params]
   (exit server))
 
-(defmacro ^:private safe-async-task [task-name & task-body]
-  `(async/thread
-     (loop []
-       (try
-         ~@task-body
-         (catch Exception e#
-           (logger/error e# (format "Error during async task %s" ~task-name))))
-       (recur))))
+(defmethod lsp.server/receive-notification "$/setTrace" [_ {:keys [server]} {:keys [value]}]
+  (lsp.server/set-trace-level server value))
+
+(defn- spawn-async-loop! [task-name ch f]
+  (async/thread
+    (loop []
+      (when-let [v (async/<!! ch)]
+        (try
+          (f v)
+          (catch Exception e
+            (logger/error e (format "Error during async task %s" task-name))))
+        (recur)))))
 
 (defn ^:private spawn-async-tasks!
   [{:keys [producer current-changes-chan diagnostics-chan
@@ -463,23 +538,23 @@
   (let [debounced-diags (shared/debounce-by diagnostics-chan diagnostics-debounce-ms :uri)
         debounced-changes (shared/debounce-by current-changes-chan change-debounce-ms :uri)
         debounced-watched-files (shared/debounce-all watched-files-chan watched-files-debounce-ms)]
-    (safe-async-task
-      :edits
-      (when-let [edit (async/<!! edits-chan)]
+    (spawn-async-loop!
+      :edits edits-chan
+      (fn [edit]
         (producer/publish-workspace-edit producer edit)))
-    (safe-async-task
-      :diagnostics
-      (when-let [diagnostic (async/<!! debounced-diags)]
+    (spawn-async-loop!
+      :diagnostics debounced-diags
+      (fn [diagnostic]
         (producer/publish-diagnostic producer diagnostic)))
-    (safe-async-task
-      :changes
-      (when-let [changes (async/<!! debounced-changes)] ;; do not put inside shared/logging-task; parked time gets included in task time
+    (spawn-async-loop!
+      :changes debounced-changes
+      (fn [changes]
         (shared/logging-task
           :analyze-file
           (f.file-management/analyze-changes changes components))))
-    (safe-async-task
-      :watched-files
-      (when-let [watched-files (async/<!! debounced-watched-files)] ;; do not put inside shared/logging-task; parked time gets included in task time
+    (spawn-async-loop!
+      :watched-files debounced-watched-files
+      (fn [watched-files]
         (shared/logging-task
           :analyze-files-in-watched-dir
           (f.file-management/analyze-watched-files! watched-files components))))))
@@ -504,7 +579,7 @@
     ;; db/db*, so it can be inspected in the nREPL.
     (alter-var-root #'db/db* (constantly db*))))
 
-(defn run-server! []
+(defn run-server! [trace-level]
   (lsp.server/discarding-stdout
     (let [timbre-logger (->TimbreLogger)
           log-path (logger/setup timbre-logger)
@@ -512,8 +587,8 @@
           db* (atom db)
           log-ch (async/chan (async/sliding-buffer 20))
           server (lsp.io-server/stdio-server {:log-ch log-ch
-                                              ;; uncomment for server-side traces
-                                              #_#_:trace-ch log-ch})
+                                              :trace-ch log-ch
+                                              :trace-level trace-level})
           producer (ClojureLspProducer. server db*)
           components {:db* db*
                       :logger timbre-logger
