@@ -1,8 +1,10 @@
 (ns clojure-lsp.feature.add-missing-libspec-test
   (:require
+   [babashka.fs :as fs]
    [clojure-lsp.feature.add-missing-libspec :as f.add-missing-libspec]
    [clojure-lsp.shared :as shared]
    [clojure-lsp.test-helper :as h]
+   [clojure.java.io :as io]
    [clojure.test :refer [deftest is testing]]
    [rewrite-clj.zip :as z]))
 
@@ -328,50 +330,81 @@
   (swap! (h/db*) shared/deep-merge {:settings (merge
                                                 {:clean {:automatically-after-ns-refactor false}}
                                                 settings)})
+
+  (h/load-java-path (str (fs/canonicalize (io/file "test" "fixtures" "java_interop" "File.java"))))
   (f.add-missing-libspec/add-missing-import (h/load-code-and-zloc code) "file:///a.clj" import-name (h/db)))
 
 (deftest add-import-to-namespace-test
   (testing "when there is no :import form"
     (is (= (h/code "(ns foo.bar "
                    "  (:import"
-                   "    java.util.Date))")
+                   "    [java.util Date]))")
            (-> "(ns foo.bar) |Date."
                (add-import-to-namespace "java.util.Date")
                as-root-str))))
   (testing "when there is no :import form with ns-inner-blocks-indentation :same-line"
     (is (= (h/code "(ns foo.bar "
-                   "  (:import java.util.Date))")
+                   "  (:import [java.util Date]))")
            (-> "(ns foo.bar) |Date."
                (add-import-to-namespace "java.util.Date"  {:clean {:ns-inner-blocks-indentation :same-line}})
                as-root-str))))
   (testing "when there is no :import form with deprecated :keep-require-at-start?"
     (is (= (h/code "(ns foo.bar "
-                   "  (:import java.util.Date))")
+                   "  (:import [java.util Date]))")
            (-> "(ns foo.bar) |Date."
                (add-import-to-namespace "java.util.Date"  {:keep-require-at-start? true})
                as-root-str))))
-  (testing "when there is a :import form already"
+  (testing "when there is a :import form already as full package import"
     (is (= (h/code "(ns foo.bar "
                    "  (:import "
-                   "    java.util.Calendar"
-                   "    java.util.Date))")
+                   "    [java.util Calendar Date]))")
            (-> (h/code "(ns foo.bar "
                        "  (:import "
                        "    java.util.Calendar)) |Date.")
                (add-import-to-namespace "java.util.Date")
                as-root-str))))
+  (testing "when there is more than one :import form already as full package import"
+    (is (= (h/code "(ns foo.bar "
+                   "  (:import "
+                   "    java.util.Calendar"
+                   "    java.util.GregorianCalendar"
+                   "    java.util.Date))")
+           (-> (h/code "(ns foo.bar "
+                       "  (:import "
+                       "    java.util.Calendar"
+                       "    java.util.GregorianCalendar)) |Date.")
+               (add-import-to-namespace "java.util.Date")
+               as-root-str))))
+  (testing "when there is a :import form already as vector import"
+    (is (= (h/code "(ns foo.bar "
+                   "  (:import "
+                   "    [java.util Calendar Date]))")
+           (-> (h/code "(ns foo.bar "
+                       "  (:import "
+                       "    [java.util Calendar])) |Date.")
+               (add-import-to-namespace "java.util.Date")
+               as-root-str))))
+  (testing "when there is a :import form already as list import"
+    (is (= (h/code "(ns foo.bar "
+                   "  (:import "
+                   "    (java.util Calendar Date)))")
+           (-> (h/code "(ns foo.bar "
+                       "  (:import "
+                       "    (java.util Calendar))) |Date.")
+               (add-import-to-namespace "java.util.Date")
+               as-root-str))))
   (testing "when there is already that :import imported"
     (is (= nil
            (-> (h/code "(ns foo.bar "
                        "  (:import "
-                       "    java.util.Date)) |Date.")
+                       "    [java.util Date])) |Date.")
                (add-import-to-namespace "java.util.Date")))))
   (testing "when there is only a :require form"
     (is (= (h/code "(ns foo.bar"
                    "  (:require"
                    "    [foo.baz :as baz]) "
                    "  (:import"
-                   "    java.util.Date))")
+                   "    [java.util Date]))")
            (-> (h/code "(ns foo.bar"
                        "  (:require"
                        "    [foo.baz :as baz])) |Date.")
@@ -382,8 +415,7 @@
                    "  (:require"
                    "    [foo.baz :as baz])"
                    "  (:import"
-                   "    java.util.Calendar"
-                   "    java.util.Date))")
+                   "    [java.util Calendar Date]))")
            (-> (h/code "(ns foo.bar"
                        "  (:require"
                        "    [foo.baz :as baz])"
@@ -394,17 +426,17 @@
   (testing "when on an invalid location"
     (is (= (h/code "(ns foo.bar "
                    "  (:import"
-                   "    java.util.Date)) ;; comment")
+                   "    [java.util Date])) ;; comment")
            (-> (h/code "(ns foo.bar) |;; comment")
                (add-import-to-namespace "java.util.Date")
                (h/changes->code (h/db)))))))
 
-(deftest add-common-import-to-namespace-test
+(deftest add-known-import-to-namespace-test
   (testing "when we known the import"
     (is (= (h/code "(ns foo.bar "
                    "  (:import"
-                   "    java.util.Date))")
-           (-> "(ns foo.bar) |Date."
+                   "    [java.io File]))")
+           (-> "(ns foo.bar) |File."
                (add-import-to-namespace nil)
                as-root-str))))
   (testing "when we don't known the import"
@@ -500,13 +532,16 @@
                             "|;; comment")
                     (add-require-suggestion "clojure.string" nil "split")))))))
 
-(defn- find-missing-import [code]
-  (f.add-missing-libspec/find-missing-import (h/zloc-from-code code)))
+(defn- find-missing-imports [code]
+  (f.add-missing-libspec/find-missing-imports (h/zloc-from-code code) (h/db)))
 
 (deftest find-missing-import-test
+  (h/load-java-path (str (fs/canonicalize (io/file "test" "fixtures" "java_interop" "File.java"))))
+  (testing "when usage is a java class"
+    (is (= '["java.io.File"] (find-missing-imports "(ns a) |File"))))
   (testing "when usage is a java constructor"
-    (is (= 'java.util.Date (find-missing-import "(ns a) |Date."))))
+    (is (= '["java.io.File"] (find-missing-imports "(ns a) (|File.)"))))
   (testing "when usage is a java ns"
-    (is (= 'java.util.Date (find-missing-import "(ns a) |Date/parse"))))
+    (is (= '["java.io.File"] (find-missing-imports "(ns a) (|File/of \"foo\")"))))
   (testing "when usage is invalid"
-    (is (nil? (find-missing-import "(ns a) |;; comment")))))
+    (is (nil? (find-missing-imports "(ns a) |;; comment")))))
